#!/bin/sh

# directory of this script
BASEDIR=$(dirname $0)

FSCONFIG="$BASEDIR/../flexiblesusy-config"
VALGRIND="`command -v valgrind`"
CMSSM_EXE="$BASEDIR/../models/CMSSM/run_CMSSM.x"
CMSSM_INPUT="$BASEDIR/../model_files/CMSSM/LesHouches.in.CMSSM"
CMSSM_OUTPUT="$BASEDIR/LesHouches.out.CMSSM"
VALGRIND_OUTPUT="${BASEDIR}/test_CMSSM_memory_leaks.valgrind.out"

if [ -z "$VALGRIND" ]; then
    echo "Error: valgrind not found"
    echo "Profiling will not be performed."
    exit 0
fi

if test ! -x "$CMSSM_EXE"; then
    echo "Error: CMSSM spectrum generator not found: $CMSSM_EXE"
    exit 1
fi

if test ! -f "$CMSSM_INPUT"; then
    echo "Error: CMSSM SLHA input file not found: $CMSSM_INPUT"
    exit 1
fi


valgrind \
    --error-exitcode=1 \
    --log-file=${VALGRIND_OUTPUT} \
    ${CMSSM_EXE} \
    --slha-input-file=${CMSSM_INPUT} \
    --slha-output-file=${CMSSM_OUTPUT}

exit_code="$?"

echo "Valgrind log file: ${VALGRIND_OUTPUT}"
echo ""
cat ${VALGRIND_OUTPUT}
echo ""

if test ${exit_code} -eq 1 ; then
    echo "Test result: FAIL"
    echo "There is a memory leak!"
else
    echo "Test result: OK"
    echo "There are no memory leaks."
fi

exit $exit_code
