
#ifndef TEST_NMSSMCPV_H
#define TEST_NMSSMCPV_H

#include "test.h"
#include "wrappers.hpp"
#include "ew_input.hpp"
#include "NMSSMCPV_two_scale_model.hpp"

using namespace flexiblesusy;

void setup_NMSSMCPV(NMSSMCPV<Two_scale>& m, const NMSSMCPV_input_parameters& input)
{
   const double ALPHASMZ = 0.1176;
   const double ALPHAMZ = 1.0 / 127.918;
   const double sinthWsq = 0.23122;
   const double alpha1 = 5 * ALPHAMZ / (3 * (1 - sinthWsq));
   const double alpha2 = ALPHAMZ / sinthWsq;
   const double g1 = sqrt(4 * Pi * alpha1);
   const double g2 = sqrt(4 * Pi * alpha2);
   const double g3 = sqrt(4 * Pi * ALPHASMZ);
   const double tanBeta = input.TanBeta;
   const double sinBeta = sin(atan(tanBeta));
   const double cosBeta = cos(atan(tanBeta));
   const double M12 = input.m12;
   const double m0 = input.m0;
   const double a0 = input.Azero;
   const double root2 = sqrt(2.0);
   const double vev = 246.0;
   const double vu = vev * sinBeta;
   const double vd = vev * cosBeta;
   const double kappa = input.KappaInput;
   const double lambda = input.LambdaInput;
   const double vS = input.LambdaTimesvSInput / lambda;
   const double scale = Electroweak_constants::MZ;

   Eigen::Matrix<std::complex<double>,3,3>
      Yu(Eigen::Matrix<std::complex<double>,3,3>::Zero()),
      Yd(Eigen::Matrix<std::complex<double>,3,3>::Zero()),
      Ye(Eigen::Matrix<std::complex<double>,3,3>::Zero()),
      mm0(Eigen::Matrix<std::complex<double>,3,3>::Zero());
   Yu(2,2) = 165.0   * root2 / (vev * sinBeta);
   Yd(2,2) = 2.9     * root2 / (vev * cosBeta);
   Ye(2,2) = 1.77699 * root2 / (vev * cosBeta);
   mm0 = Sqr(m0) * Eigen::Matrix<std::complex<double>,3,3>::Identity();

   m.set_input_parameters(input);
   m.set_scale(scale);
   m.set_loops(1);
   m.set_thresholds(3);
   m.set_g1(g1);
   m.set_g2(g2);
   m.set_g3(g3);
   m.set_Yu(Yu);
   m.set_Yd(Yd);
   m.set_Ye(Ye);
   m.set_MassB(M12);
   m.set_MassG(M12);
   m.set_MassWB(M12);
   m.set_mq2(mm0);
   m.set_ml2(mm0);
   m.set_md2(mm0);
   m.set_mu2(mm0);
   m.set_me2(mm0);
   m.set_mHd2(Sqr(m0));
   m.set_mHu2(Sqr(m0));
   m.set_ms2(Sqr(m0));
   m.set_TYu(a0 * Yu);
   m.set_TYd(a0 * Yd);
   m.set_TYe(a0 * Ye);
   m.set_TKappa(kappa);
   m.set_TLambdax(lambda);
   m.set_Kappa(kappa);
   m.set_Lambdax(lambda);
   m.set_vu(vu);
   m.set_vd(vd);
   m.set_vS(vS);
   m.set_eta(input.etaInput);
   m.set_etaS(input.etaSInput);
}

#endif
